package Mojar::Mysql;
use strict;
use version;
use 5.010001;

our $VERSION = 2.001;

1
__END__

=head1 NAME

Mojar::Mysql - Suite of MySQL integration tools.

=head1 DESCRIPTION

This initial release contains the database connector L<Mojar::Mysql::Connector>.

=cut
